package chart_theme

import (
	"eta_gn/eta_chart_lib/global"
	"time"
)

type ChartThemeType struct {
	ChartThemeTypeId    int       `description:"图表主题类型ID" orm:"column(chart_theme_type_id);pk" gorm:"column:chart_theme_type_id;primaryKey"`
	ChartTypeName       string    `description:"类型名称" gorm:"column:chart_type_name"`
	ChartType           int       `description:"图表类型" gorm:"column:chart_type"`
	ChartSource         int       `description:"图表来源" gorm:"column:chart_source"`
	DefaultChartThemeId int       `description:"默认使用的主题id" gorm:"column:default_chart_theme_id"`
	ModifyTime          time.Time `description:"修改时间" gorm:"column:modify_time"`
	CreateTime          time.Time `description:"创建时间" gorm:"column:create_time"`
}

func (ChartThemeType) TableName() string {
	return "chart_theme_type"
}


func (m *ChartThemeType) Update(cols []string) (err error) {
	o := global.DmSQL["data"]
	err = o.Model(m).Select(cols).Updates(m).Error
	return
}



func GetAllChartThemeTypeList() (list []*ChartThemeType, err error) {
	o := global.DmSQL["data"]
	sql := `SELECT * FROM chart_theme_type ORDER BY chart_theme_type_id ASC `
	err = o.Raw(sql).Scan(&list).Error
	return
}



func GetChartThemeTypeListBySource(source int) (list []*ChartThemeType, err error) {
	o := global.DmSQL["data"]
	sql := `SELECT * FROM chart_theme_type WHERE chart_source = ?  ORDER BY chart_theme_type_id ASC `
	err = o.Raw(sql, source).Scan(&list).Error
	return
}



func GetChartThemeTypeById(chartThemeTypeId int) (item *ChartThemeType, err error) {
	o := global.DmSQL["data"]
	sql := `SELECT * FROM chart_theme_type where chart_theme_type_id = ? `
	err = o.Raw(sql, chartThemeTypeId).First(&item).Error
	return
}



func GetChartThemeTypeByChartTypeAndSource(chartType, source int) (item *ChartThemeType, err error) {
	o := global.DmSQL["data"]
	sql := `SELECT * FROM chart_theme_type where chart_type = ? AND chart_source = ? `
	err = o.Raw(sql, chartType, source).First(&item).Error
	return
}
