package cross_variety

import (
	"eta_gn/eta_chart_lib/global"
	"eta_gn/eta_chart_lib/utils"
	"time"
)

type ChartVariety struct {
	ChartVarietyId     int       `orm:"column(chart_variety_id);pk" gorm:"column:chart_variety_id;primaryKey"`
	ChartVarietyName   string    `description:"品种名称" gorm:"column:chart_variety_name"`
	ChartVarietyNameEn string    `description:"品种名称（英文）" gorm:"column:chart_variety_name_en"`
	SysUserId          int       `description:"创建人id" gorm:"column:sys_user_id"`
	SysUserRealName    string    `description:"创建人姓名" gorm:"column:sys_user_real_name"`
	ModifyTime         time.Time `description:"修改时间" gorm:"column:modify_time"`
	CreateTime         time.Time `description:"创建时间" gorm:"column:create_time"`
}

func (ChartVariety) TableName() string {
	return "chart_variety"
}




func GetVarietyById(id int) (item *ChartVariety, err error) {
	o := global.DmSQL["data"]
	sql := `SELECT * FROM chart_variety WHERE chart_variety_id = ?`
	err = o.Raw(sql, id).First(&item).Error
	return
}



func GetVarietyByName(name string) (item *ChartVariety, err error) {
	o := global.DmSQL["data"]
	sql := `SELECT * FROM chart_variety WHERE chart_variety_name = ?`
	err = o.Raw(sql, name).First(&item).Error
	return
}




func AddVariety(item *ChartVariety) (err error) {
	o := global.DmSQL["data"]
	err = o.Create(item).Error
	return
}



func (item *ChartVariety) Update(updateColList []string) (err error) {
	to := global.DmSQL["data"]
	err = to.Model(item).Select(updateColList).Updates(item).Error
	return
}


func (item *ChartVariety) Delete() (err error) {
	o := global.DmSQL["data"]
	err = o.Delete(item).Error
	return
}



func GetVarietyList() (items []*ChartVariety, err error) {
	o := global.DmSQL["data"]
	sql := `SELECT * FROM chart_variety WHERE 1 = 1 `
	err = o.Raw(sql).Scan(&items).Error
	return
}

func GetVarietyListByIdList(idList []int) (items []*ChartVariety, err error) {
	num := len(idList)
	if num <= 0 {
		return
	}
	o := global.DmSQL["data"]
	sql := `SELECT * FROM chart_variety WHERE 1 = 1 AND chart_variety_id in (` + utils.GetGormInReplace(num) + `)`
	err = o.Raw(sql, idList).Scan(&items).Error
	return
}
