package excel

import (
	"eta_gn/eta_chart_lib/global"
	"eta_gn/eta_chart_lib/utils"
	"fmt"
	"time"
)

type ExcelEdbMapping struct {
	ExcelEdbMappingId int       `gorm:"column:excel_edb_mapping_id;primaryKey"`
	ExcelInfoId       int       `gorm:"column:excel_info_id" description:"excel的id"`
	Source            int       `gorm:"column:source" description:"表格来源，1：excel插件的表格，2：自定义表格，3：混合表格，4：自定义分析，默认：1"`
	EdbInfoId         int       `gorm:"column:edb_info_id" description:"计算指标id"`
	CreateTime        time.Time `gorm:"column:create_time" description:"创建时间"`
	ModifyTime        time.Time `gorm:"column:modify_time" description:"修改时间"`
}

func AddExcelEdbMappingMulti(items []*ExcelEdbMapping) (err error) {
	o := global.DmSQL["data"]
	err = o.CreateInBatches(items, utils.MultiAddNum).Error
	return
}

func (e *ExcelEdbMapping) Add() (err error) {
	o := global.DmSQL["data"]
	err = o.Create(e).Error
	return
}

func GetExcelEdbMappingByEdbInfoId(edbInfoId int) (item *ExcelEdbMapping, err error) {
	o := global.DmSQL["data"]
	sql := ` SELECT *  FROM excel_edb_mapping WHERE 1=1 AND edb_info_id = ? `

	err = o.Raw(sql, edbInfoId).First(&item).Error
	return
}

func GetExcelEdbMappingByExcelInfoId(excelInfoId int) (items []*ExcelEdbMapping, err error) {
	o := global.DmSQL["data"]
	sql := ` SELECT *  FROM excel_edb_mapping AS a 
           join edb_info as b on a.edb_info_id = b.edb_info_id
           WHERE 1=1 AND a.excel_info_id = ? `
	err = o.Raw(sql, excelInfoId).Scan(&items).Error

	return
}







type ExcelEdbMappingItem struct {
	EdbInfoId        int    `gorm:"column:edb_info_id" description:"指标id"`
	UniqueCode       string `gorm:"column:unique_code" description:"唯一编码"`
	EdbName          string `gorm:"column:edb_name" description:"指标名称"`
	ClassifyId       int    `gorm:"column:classify_id" description:"分类id"`
	Frequency        string `gorm:"column:frequency" description:"频度"`
	Unit             string `gorm:"column:unit" description:"单位"`
	CalculateFormula string `gorm:"column:calculate_formula" json:"-"` // 隐藏该字段
	DateSequenceStr  string `gorm:"column:date_sequence_str" description:"日期序列公式"`
	DataSequenceStr  string `gorm:"column:data_sequence_str" description:"数据序列公式"`
}

type CalculateFormula struct {
	DateSequenceStr string `json:"DateSequenceStr"`
	DataSequenceStr string `json:"DataSequenceStr"`
}

func GetAllExcelEdbMappingItemByExcelInfoId(excelInfoId int) (items []*ExcelEdbMappingItem, err error) {
	o := global.DmSQL["data"]
	sql := `SELECT a.edb_info_id,a.unique_code,a.edb_name,a.classify_id,a.frequency,a.unit,calculate_formula FROM edb_info AS a 
         JOIN excel_edb_mapping AS b ON a.edb_info_id=b.edb_info_id 
         WHERE b.excel_info_id = ? ORDER BY b.excel_edb_mapping_id ASC `
	err = o.Raw(sql, excelInfoId).Scan(&items).Error
	return
}

func GetNoCustomAnalysisExcelEdbMappingCount(edbInfoId int) (count int, err error) {
	o := global.DmSQL["data"]
	sql := ` SELECT COUNT(1) AS count FROM excel_edb_mapping a 
                          join excel_info b on a.excel_info_id=b.excel_info_id
                          WHERE edb_info_id=? AND a.source != 4 AND b.is_delete = 0`
	err = o.Raw(sql, edbInfoId).Scan(&count).Error
	return
}

func GetAllExcelEdbMappingByExcelInfoId(excelInfoId int) (items []*ExcelEdbMapping, err error) {
	o := global.DmSQL["data"]
	sql := `SELECT a.* FROM  excel_edb_mapping a
         WHERE a.excel_info_id = ? ORDER BY a.excel_edb_mapping_id ASC `
	err = o.Raw(sql, excelInfoId).Scan(&items).Error
	return
}

func DeleteCustomAnalysisExcelEdbMappingByEdbInfoId(excelInfoId int) (err error) {
	o := global.DmSQL["data"]
	sql := `DELETE FROM excel_edb_mapping WHERE source = ? AND edb_info_id = ? LIMIT 1`
	err = o.Exec(sql, utils.CUSTOM_ANALYSIS_TABLE, excelInfoId).Error

	return
}

func GetExcelEdbMappingItemByExcelInfoIdOrKeyword(excelInfoId int, keyword string) (items []*ExcelEdbMappingItem, err error) {
	o := global.DmSQL["data"]
	cond := `b.excel_info_id = ?`
	pars := make([]interface{}, 0)
	pars = append(pars, excelInfoId)
	if keyword != "" {
		cond += ` AND (a.edb_code LIKE ? OR a.edb_name LIKE ?)`
		pars = append(pars, keyword, keyword)
	}
	sql := fmt.Sprintf(`SELECT
			a.edb_info_id,
			a.unique_code,
			a.edb_name,
			a.classify_id,
			a.frequency,
			a.unit,
			calculate_formula
		FROM
			edb_info AS a
		JOIN excel_edb_mapping AS b ON a.edb_info_id = b.edb_info_id
		WHERE
			%s
		ORDER BY
			b.excel_edb_mapping_id ASC`, cond)
	err = o.Raw(sql, pars...).Scan(&items).Error
	return
}






