package request

type MoveExcelInfoReq struct {
	ExcelClassifyId int `description:"excel表格分类id"`
	ExcelInfoId     int `description:"excel表格ID"`
	PrevExcelInfoId int `description:"上一个excel表格ID"`
	NextExcelInfoId int `description:"下一个excel表格ID"`
}

type DeleteExcelInfoReq struct {
	ExcelInfoId int `description:"表格ID"`
}

type AddExcelInfoReq struct {
	ExcelInfoId     int         `description:"表格ID"`
	ExcelName       string      `description:"表格名称"`
	Source          int         `description:"表格来源，1：excel插件的表格，2：自定义表格，默认：1"`
	ExcelType       int         `description:"表格类型，1：指标列，2：日期列，默认：1"`
	ExcelImage      string      `description:"表格截图"`
	ExcelClassifyId int         `description:"分类id"`
	Content         string      `description:"Excel表格内容"`
	TableData       interface{} `description:"自定义表格的数据内容"`
}

type EditExcelInfoReq struct {
	ExcelInfoId     int         `description:"ETA表格ID"`
	ExcelType       int         `description:"表格类型，1：指标列，2：日期列，默认：1"`
	ExcelName       string      `description:"表格名称"`
	ExcelImage      string      `description:"表格截图"`
	ExcelClassifyId int         `description:"分类id"`
	Content         string      `description:"Excel表格内容"`
	TableData       interface{} `description:"自定义表格的数据内容"`
}

type SetExcelInfoImageReq struct {
	ExcelInfoId int    `description:"表格ID"`
	ImageUrl    string `description:"表格图片地址"`
}

type AddExcelDraftReq struct {
	ExcelInfoId     int    `description:"ETA表格ID"`
	ExcelName       string `description:"表格名称"`
	ExcelClassifyId int    `description:"分类id"`
	Content         string `description:"Excel表格内容"`
}

type CalculateReq struct {
	CalculateFormula string             `description:"计算公式"`
	TagMap           map[string]float64 `description:"标签与值的关系"`
}

type GetOtherEdbDateDataReq struct {
	EdbInfoId int      `description:"指标id"`
	SortType  string   `description:"如何排序，是正序还是倒序，枚举值：asc 正序,desc 倒序，不传默认倒序"`
	DateList  []string `description:"日期列表，从小到大"`
}

type GetFutureDateDataReq struct {
	EdbInfoIdList []int  `description:"指标id列表，从左至右，从上到下的顺序"`
	DateType      int    `description:"日期类型，1：期数，2：截止日期"`
	Num           int    `description:"需要添加的期数"`
	StartDate     string `description:"开始日期"`
	EndDate       string `description:"结束日期"`
}

type GetHistoryDateDataReq struct {
	EdbInfoIdList []int  `description:"指标id列表，从左至右，从上到下的顺序"`
	Num           int    `description:"需要添加的期数"`
	EndDate       string `description:"结束日期"`
}

type TableSaveReq struct {
	ExcelName       string      `description:"表格名称"`
	ExcelImage      string      `description:"表格截图"`
	ExcelClassifyId int         `description:"分类id"`
	TableData       interface{} `description:"表格数据"`
}

type TableDataReq struct {
	EdbInfoIdList []int             `description:"指标id列表，从左至右，从上到下的顺序"`
	Sort          int               `description:"日期排序，1：倒序，2：正序"`
	Data          []EdbInfoData     `description:"数据列表"`
	TextRowData   [][]ManualDataReq `description:"文本列表"`
}

type EdbInfoData struct {
	EdbInfoId        int             `description:"指标ID"`
	Tag              string          `description:"标签"`
	EdbName          string          `description:"指标名称"`
	EdbNameEn        string          `description:"英文指标名称"`
	EdbAliasName     string          `description:"指标别名"`
	Frequency        string          `description:"频度"`
	Unit             string          `description:"单位"`
	UnitEn           string          `description:"英文单位"`
	ClassifyId       int             `description:"所属分类" json:"-"`
	IsJoinPermission int             `description:"是否加入权限管控，0：不加入；1：加入；默认：0" json:"-"`
	HaveOperaAuth    bool            `description:"是否有数据权限，默认：false"`
	Data             []ManualDataReq `description:"单元格数据列表"`
}

type ManualDataReq struct {
	DataType            int               `description:"数据类型，1：普通的，2：插值法，3：手动输入，4：公式计算，5：预测值"`
	DataTime            string            `description:"所属日期"`
	DataTimeType        int               `description:"日期类型，1：实际日期；2：未来日期"`
	ShowValue           string            `description:"展示值"`
	Value               string            `description:"实际值（计算公式）"`
	RelationEdbInfoList []RelationEdbInfo `description:"关联指标（计算公式中关联的指标，用于计算的时候去匹配）"`
}

type RelationEdbInfo struct {
	Tag string `description:"指标标签"`
	Row string `description:"第几行"`
}

type CopyExcelInfoReq struct {
	ExcelInfoId     int    `description:"ETA表格ID"`
	ExcelName       string `description:"表格名称"`
	ExcelClassifyId int    `description:"分类id"`
}

type MarkEditExcel struct {
	ExcelInfoId int `description:"表格id"`
	Status      int `description:"标记状态，1：编辑中，2：编辑完成"`
}
