package data_manage

import (
	"eta_gn/eta_chart_lib/global"
	"eta_gn/eta_chart_lib/utils"
	"fmt"
	"strings"
	"time"
)


type FactorEdbSeriesMapping struct {
	FactorEdbSeriesMappingId int       `gorm:"column:factor_edb_series_mapping_id;primaryKey" description:"因子指标系列映射ID" orm:"column(factor_edb_series_mapping_id);pk"`
	FactorEdbSeriesId        int       `gorm:"column:factor_edb_series_id" description:"因子指标系列ID"`
	EdbInfoId                int       `gorm:"column:edb_info_id" description:"指标ID"`
	EdbCode                  string    `gorm:"column:edb_code" description:"指标编码"`
	CreateTime               time.Time `gorm:"column:create_time" description:"创建时间"`
	ModifyTime               time.Time `gorm:"column:modify_time" description:"修改时间"`
}

func (m *FactorEdbSeriesMapping) TableName() string {
	return "factor_edb_series_mapping"
}

type FactorEdbSeriesMappingCols struct {
	PrimaryId         string
	FactorEdbSeriesId string
	EdbInfoId         string
	EdbCode           string
	CreateTime        string
	ModifyTime        string
}

func (m *FactorEdbSeriesMapping) Cols() FactorEdbSeriesMappingCols {
	return FactorEdbSeriesMappingCols{
		PrimaryId:         "factor_edb_series_mapping_id",
		FactorEdbSeriesId: "factor_edb_series_id",
		EdbInfoId:         "edb_info_id",
		EdbCode:           "edb_code",
		CreateTime:        "create_time",
		ModifyTime:        "modify_time",
	}
}

func (m *FactorEdbSeriesMapping) Create() (err error) {
	o := global.DmSQL["data"]
	err = o.Create(m).Error
	return
}

func (m *FactorEdbSeriesMapping) CreateMulti(items []*FactorEdbSeriesMapping) (err error) {
	if len(items) == 0 {
		return
	}
	o := global.DmSQL["data"]
	err = o.CreateInBatches(items, utils.MultiAddNum).Error
	return
}

func (m *FactorEdbSeriesMapping) Update(cols []string) (err error) {
	o := global.DmSQL["data"]
	err = o.Model(m).Select(cols).Updates(m).Error
	return
}

func (m *FactorEdbSeriesMapping) Remove() (err error) {
	o := global.DmSQL["data"]
	sql := fmt.Sprintf(`DELETE FROM %s WHERE %s = ? LIMIT 1`, m.TableName(), m.Cols().PrimaryId)
	err = o.Exec(sql, m.FactorEdbSeriesMappingId).Error
	return
}

func (m *FactorEdbSeriesMapping) MultiRemove(ids []int) (err error) {
	if len(ids) == 0 {
		return
	}
	o := global.DmSQL["data"]
	sql := fmt.Sprintf(`DELETE FROM %s WHERE %s IN (%s)`, m.TableName(), m.Cols().PrimaryId, utils.GetGormInReplace(len(ids)))
	err = o.Exec(sql, ids).Error
	return
}

func (m *FactorEdbSeriesMapping) RemoveByCondition(condition string, pars []interface{}) (err error) {
	if condition == "" {
		return
	}
	o := global.DmSQL["data"]
	sql := fmt.Sprintf(`DELETE FROM %s WHERE %s`, m.TableName(), condition)
	err = o.Exec(sql, pars...).Error
	return
}

func (m *FactorEdbSeriesMapping) GetItemById(id int) (item *FactorEdbSeriesMapping, err error) {
	o := global.DmSQL["data"]
	sql := fmt.Sprintf(`SELECT * FROM %s WHERE %s = ? LIMIT 1`, m.TableName(), m.Cols().PrimaryId)
	err = o.Raw(sql, id).First(&item).Error
	return
}

func (m *FactorEdbSeriesMapping) GetItemByCondition(condition string, pars []interface{}, orderRule string) (item *FactorEdbSeriesMapping, err error) {
	o := global.DmSQL["data"]
	order := ``
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT * FROM %s WHERE 1=1 %s %s LIMIT 1`, m.TableName(), condition, order)
	err = o.Raw(sql, pars...).First(&item).Error
	return
}

func (m *FactorEdbSeriesMapping) GetCountByCondition(condition string, pars []interface{}) (count int, err error) {
	o := global.DmSQL["data"]
	sql := fmt.Sprintf(`SELECT COUNT(1) FROM %s WHERE 1=1 %s`, m.TableName(), condition)
	err = o.Raw(sql, pars...).Scan(&count).Error
	return
}

func (m *FactorEdbSeriesMapping) GetItemsByCondition(condition string, pars []interface{}, fieldArr []string, orderRule string) (items []*FactorEdbSeriesMapping, err error) {
	o := global.DmSQL["data"]
	fields := strings.Join(fieldArr, ",")
	if len(fieldArr) == 0 {
		fields = `*`
	}
	order := fmt.Sprintf(`ORDER BY %s DESC`, m.Cols().CreateTime)
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT %s FROM %s WHERE 1=1 %s %s`, fields, m.TableName(), condition, order)
	err = o.Raw(sql, pars...).Scan(&items).Error
	return
}

func (m *FactorEdbSeriesMapping) GetPageItemsByCondition(condition string, pars []interface{}, fieldArr []string, orderRule string, startSize, pageSize int) (items []*FactorEdbSeriesMapping, err error) {
	o := global.DmSQL["data"]
	fields := strings.Join(fieldArr, ",")
	if len(fieldArr) == 0 {
		fields = `*`
	}
	order := fmt.Sprintf(`ORDER BY %s DESC`, m.Cols().CreateTime)
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT %s FROM %s WHERE 1=1 %s %s LIMIT ?,?`, fields, m.TableName(), condition, order)
	pars = append(pars, startSize, pageSize)
	err = o.Raw(sql, pars...).Scan(&items).Error
	return
}













type FactorEdbSeriesMappingItem struct {
	SeriesId  int    `gorm:"column:series_id" description:"因子指标系列ID"`
	EdbInfoId int    `gorm:"column:edb_info_id" description:"指标ID"`
	EdbCode   string `gorm:"column:edb_code" description:"指标编码"`
	EdbName   string `gorm:"column:edb_name" description:"指标名称"`
	EdbNameEn string `gorm:"column:edb_name_en" description:"指标名称-英文"`
}

func (m *FactorEdbSeriesMapping) Format2Item() (item *FactorEdbSeriesMappingItem) {
	item = new(FactorEdbSeriesMappingItem)
	item.SeriesId = m.FactorEdbSeriesId
	item.EdbInfoId = m.EdbInfoId
	item.EdbCode = m.EdbCode
	return
}


func GetChartUsedFactorSeriesEdb(chartId int) (items []*FactorEdbSeriesMapping, err error) {
	o := global.DmSQL["data"]
	chartOb := new(FactorEdbSeriesChartMapping)
	edbOb := new(FactorEdbSeriesMapping)
	sql := fmt.Sprintf(`SELECT b.* FROM %s AS a
	JOIN %s AS b ON a.%s = b.%s AND a.%s = b.%s
	WHERE a.%s = ? ORDER BY %s ASC`, chartOb.TableName(), edbOb.TableName(), chartOb.Cols().FactorEdbSeriesId, edbOb.Cols().FactorEdbSeriesId, chartOb.Cols().EdbInfoId, edbOb.Cols().EdbInfoId, chartOb.Cols().ChartInfoId, edbOb.Cols().FactorEdbSeriesId)
	err = o.Raw(sql, chartId).Scan(&items).Error
	return
}

