package future_good

import (
	"eta_gn/eta_chart_lib/global"
	"fmt"
	"time"
)


type FutureGoodChartClassify struct {
	FutureGoodChartClassifyId   int       `gorm:"column:future_good_chart_classify_id;primaryKey" description:"分类ID" orm:"column(future_good_chart_classify_id);pk"`
	FutureGoodChartClassifyName string    `gorm:"column:future_good_chart_classify_name" description:"分类名称"`
	ParentId                    int       `gorm:"column:parent_id" description:"父级id"`
	SysUserId                   int       `gorm:"column:sys_user_id" description:"创建人id"`
	SysUserRealName             string    `gorm:"column:sys_user_real_name" description:"创建人姓名"`
	Level                       int       `gorm:"column:level" description:"层级"`
	UniqueCode                  string    `gorm:"column:unique_code" description:"唯一编码"`
	Sort                        int       `gorm:"column:sort" description:"排序字段，越小越靠前，默认值：10"`
	IsDelete                    int       `gorm:"column:is_delete" description:"是否删除，0：未删除，1：已删除"`
	CreateTime                  time.Time `gorm:"column:create_time" description:"创建时间"`
	ModifyTime                  time.Time `gorm:"column:modify_time" description:"修改时间"`
}

func AddFutureGoodChartClassify(item *FutureGoodChartClassify) (lastId int64, err error) {
	o := global.DmSQL["data"]
	err = o.Create(item).Error
	return
}

func GetFutureGoodChartClassifyCount(FutureGoodChartClassifyName string, parentId int) (count int, err error) {
	o := global.DmSQL["data"]
	sql := `SELECT COUNT(1) AS count FROM future_good_chart_classify WHERE parent_id=? AND future_good_chart_classify_name=? AND is_delete=0 `
	err = o.Raw(sql, parentId, FutureGoodChartClassifyName).Scan(&count).Error
	return
}

func GetFutureGoodChartClassifyById(classifyId int) (item *FutureGoodChartClassify, err error) {
	o := global.DmSQL["data"]
	sql := `SELECT * FROM future_good_chart_classify WHERE future_good_chart_classify_id=? AND is_delete=0 `
	err = o.Raw(sql, classifyId).First(&item).Error
	return
}

func GetFutureGoodChartClassifyByParentId(parentId int) (items []*FutureGoodChartClassifyItems, err error) {
	o := global.DmSQL["data"]
	sql := ` SELECT * FROM future_good_chart_classify WHERE parent_id=? AND is_delete=0 order by sort asc,future_good_chart_classify_id asc`
	err = o.Raw(sql, parentId).Scan(&items).Error
	return
}

func GetFutureGoodChartClassifyAll() (items []*FutureGoodChartClassifyItems, err error) {
	o := global.DmSQL["data"]
	sql := ` SELECT * FROM future_good_chart_classify WHERE parent_id<>0 AND is_delete=0 order by sort asc,future_good_chart_classify_id asc`
	err = o.Raw(sql).Scan(&items).Error
	return
}






type FutureGoodChartClassifyItems struct {
	FutureGoodChartClassifyId   int                             `gorm:"column:future_good_chart_classify_id" description:"分类id"`
	FutureGoodChartInfoId       int                             `gorm:"column:future_good_chart_info_id" description:"表格id"`
	FutureGoodChartClassifyName string                          `gorm:"column:future_good_chart_classify_name"`
	ParentId                    int                             `gorm:"column:parent_id"`
	Level                       int                             `gorm:"column:level" description:"层级"`
	Sort                        int                             `gorm:"column:sort" description:"排序字段，越小越靠前，默认值：10"`
	UniqueCode                  string                          `gorm:"column:unique_code" description:"唯一编码"`
	SysUserId                   int                             `gorm:"column:sys_user_id" description:"创建人id"`
	SysUserRealName             string                          `gorm:"column:sys_user_real_name" description:"创建人姓名"`
	StartDate                   string                          `gorm:"column:start_date" description:"自定义开始日期"`
	Children                    []*FutureGoodChartClassifyItems `description:"子分类列表"`
}

func GetFutureGoodChartClassifyByCondition(condition string, pars []interface{}) (item *FutureGoodChartClassify, err error) {
	o := global.DmSQL["data"]
	sql := ` SELECT * FROM future_good_chart_classify WHERE 1=1 AND is_delete=0 `
	if condition != "" {
		sql += condition
	}
	err = o.Raw(sql, pars...).First(&item).Error
	return
}

func GetNextFutureGoodChartClassifyByCondition(condition string, pars []interface{}) (item *FutureGoodChartClassify, err error) {
	o := global.DmSQL["data"]
	sql := ` SELECT * FROM future_good_chart_classify WHERE 1=1 AND is_delete=0 `
	if condition != "" {
		sql += condition
	}
	sql += " ORDER BY sort asc , create_time ASC LIMIT 1 "
	err = o.Raw(sql, pars...).First(&item).Error
	return
}

func GetFirstFutureGoodChartClassifyByParentId(parentId int) (item *FutureGoodChartClassify, err error) {
	o := global.DmSQL["data"]
	sql := ` SELECT * FROM future_good_chart_classify WHERE parent_id=? AND is_delete=0 order by sort asc,future_good_chart_classify_id asc limit 1`
	err = o.Raw(sql, parentId).First(&item).Error
	return
}

func UpdateFutureGoodChartClassifySortByParentId(parentId, classifyId, nowSort int, updateSort string) (err error) {
	o := global.DmSQL["data"]
	sql := ` update future_good_chart_classify set sort = ` + updateSort + ` WHERE parent_id=? and sort > ? AND is_delete=0 `
	if classifyId > 0 {
		sql += ` or ( future_good_chart_classify_id > ` + fmt.Sprint(classifyId) + ` and sort= ` + fmt.Sprint(nowSort) + `)`
	}
	err = o.Exec(sql, parentId, nowSort).Error
	return
}

func (FutureGoodChartClassify *FutureGoodChartClassify) Update(cols []string) (err error) {
	o := global.DmSQL["data"]
	err = o.Model(FutureGoodChartClassify).Select(cols).Updates(FutureGoodChartClassify).Error
	return
}

func GetFutureGoodChartClassifyMaxSort(parentId int) (sort int, err error) {
	o := global.DmSQL["data"]
	sql := `SELECT COALESCE(MAX(sort), 0) AS sort FROM future_good_chart_classify WHERE parent_id=? AND is_delete=0 `
	err = o.Raw(sql, parentId).Scan(&sort).Error
	return
}







type FutureGoodChartClassifyView struct {
	FutureGoodChartClassifyId   int    `gorm:"column:future_good_chart_classify_id;primaryKey" description:"分类ID" orm:"column(future_good_chart_classify_id);pk"`
	FutureGoodChartClassifyName string `gorm:"column:future_good_chart_classify_name" description:"分类名称"`
	ParentId                    int    `gorm:"column:parent_id" description:"父级id"`
}

func GetFutureGoodChartClassifyViewById(classifyId int) (item *FutureGoodChartClassifyView, err error) {
	o := global.DmSQL["data"]
	sql := `SELECT * FROM future_good_chart_classify WHERE future_good_chart_classify_id=? AND is_delete=0 `
	err = o.Raw(sql, classifyId).First(&item).Error
	return
}

func GetChartInfoAll() (items []*FutureGoodChartClassifyItems, err error) {
	o := global.DmSQL["data"]
	sql := ` SELECT chart_info_id,chart_classify_id,chart_name AS chart_classify_name,
             unique_code,sys_user_id,sys_user_real_name,date_type,start_date,end_date,chart_type,calendar,season_start_date,season_end_date
            FROM chart_info ORDER BY sort asc,create_time ASC `
	err = o.Raw(sql).Scan(&items).Error
	return
}

func GetNoContentFutureGoodInfoAll() (items []FutureGoodChartClassifyItems, err error) {
	o := global.DmSQL["data"]
	sql := ` SELECT excel_info_id,excel_classify_id,excel_name AS excel_classify_name,
             unique_code,sys_user_id,sys_user_real_name
            FROM excel_info where is_delete=0 ORDER BY sort asc,create_time desc `
	err = o.Raw(sql).Scan(&items).Error
	return
}

func GetFutureGoodChartInfoCountByClassifyId(classifyId int) (total int64, err error) {
	o := global.DmSQL["data"]
	sql := ` SELECT count(1) total FROM excel_info WHERE excel_classify_id = ? AND is_delete=0 `
	err = o.Raw(sql, classifyId).Scan(&total).Error
	return
}




