package data_manage

import (
	"eta_gn/eta_chart_lib/global"
	"eta_gn/eta_chart_lib/utils"
	"time"
)

type PredictEdbConfCalculateMapping struct {
	PredictEdbConfCalculateMappingId int       `gorm:"column:predict_edb_conf_calculate_mapping_id;primaryKey" description:"预测指标计算映射ID" orm:"column(predict_edb_conf_calculate_mapping_id);pk"`
	EdbInfoId                        int       `gorm:"column:edb_info_id" description:"指标id"`
	ConfigId                         int       `gorm:"column:config_id" description:"配置id"`
	FromEdbInfoId                    int       `gorm:"column:from_edb_info_id" description:"基础指标id"`
	FromEdbCode                      string    `gorm:"column:from_edb_code" description:"基础指标编码"`
	FromEdbName                      string    `gorm:"column:from_edb_name" description:"基础指标名称"`
	FromSource                       int       `gorm:"column:from_source" description:"基础指标来源"`
	FromSourceName                   string    `gorm:"column:from_source_name" description:"基础指标来源名称"`
	FromTag                          string    `gorm:"column:from_tag" description:"来源指标标签"`
	Sort                             int       `gorm:"column:sort" description:"计算指标名称排序"`
	CreateTime                       time.Time `gorm:"column:create_time" description:"创建时间"`
	ModifyTime                       time.Time `gorm:"column:modify_time" description:"修改时间"`
}

func GetPredictEdbConfCalculateMappingListById(edbInfoId int) (items []*PredictEdbConf, err error) {
	o := global.DmSQL["data"]
	sql := ` SELECT * FROM predict_edb_conf_calculate_mapping WHERE edb_info_id=? ORDER BY predict_edb_conf_calculate_mapping_id ASC`
	err = o.Raw(sql, edbInfoId).Scan(&items).Error
	return
}

func GetPredictEdbConfCalculateMappingListByConfigId(edbInfoId, configId int) (items []*PredictEdbConf, err error) {
	o := global.DmSQL["data"]
	sql := ` SELECT * FROM predict_edb_conf_calculate_mapping WHERE edb_info_id=? AND config_id=? ORDER BY predict_edb_conf_calculate_mapping_id ASC`
	err = o.Raw(sql, edbInfoId, configId).Scan(&items).Error
	return
}



type PredictEdbConfCalculateMappingDetail struct {
	PredictEdbConfCalculateMappingId int       `gorm:"column:predict_edb_conf_calculate_mapping_id;primaryKey" description:"预测指标计算映射ID" orm:"column(predict_edb_conf_calculate_mapping_id);pk"`
	EdbInfoId                        int       `gorm:"column:edb_info_id" description:"指标id"`
	ConfigId                         int       `gorm:"column:config_id" description:"配置id"`
	FromEdbInfoId                    int       `gorm:"column:from_edb_info_id" description:"基础指标id"`
	FromEdbCode                      string    `gorm:"column:from_edb_code" description:"基础指标编码"`
	FromEdbName                      string    `gorm:"column:from_edb_name" description:"基础指标名称"`
	FromSource                       int       `gorm:"column:from_source" description:"基础指标来源"`
	FromSourceName                   string    `gorm:"column:from_source_name" description:"基础指标来源名称"`
	FromTag                          string    `gorm:"column:from_tag" description:"来源指标标签"`
	Sort                             int       `gorm:"column:sort" description:"计算指标名称排序"`
	CreateTime                       time.Time `gorm:"column:create_time" description:"创建时间"`
	ModifyTime                       time.Time `gorm:"column:modify_time" description:"修改时间"`
	StartDate                        string    `gorm:"column:start_date" description:"开始日期"`
	EndDate                          string    `gorm:"column:end_date" description:"结束日期"`
	EdbType                          int       `gorm:"column:edb_type" description:"指标类型:1:基础指标,2:计算指标"`
	EdbCode                          string    `gorm:"column:edb_code" description:"指标code"`
}

func GetPredictEdbConfCalculateMappingDetailListById(edbInfoId int) (list []*PredictEdbConfCalculateMappingDetail, err error) {
	o := global.DmSQL["data"]
	sql := ` SELECT a.predict_edb_conf_calculate_mapping_id,a.edb_info_id,a.from_edb_info_id,a.from_edb_code,a.from_source,a.from_source_name,a.sort,a.create_time,a.modify_time,a.from_tag,b.edb_name_source as from_edb_name,b.start_date,b.end_date,b.edb_type FROM predict_edb_conf_calculate_mapping AS a
			INNER JOIN edb_info AS b ON a.from_edb_info_id=b.edb_info_id
			WHERE a.edb_info_id=? ORDER BY sort ASC `

	err = o.Raw(sql, edbInfoId).Scan(&list).Error
	return
}

func GetPredictEdbConfCalculateMappingDetailListByConfigId(edbInfoId, configId int) (list []*PredictEdbConfCalculateMappingDetail, err error) {
	o := global.DmSQL["data"]
	sql := ` SELECT a.predict_edb_conf_calculate_mapping_id,a.edb_info_id,a.from_edb_info_id,a.from_edb_code,a.from_source,a.from_source_name,a.sort,a.create_time,a.modify_time,a.from_tag,b.edb_name_source as from_edb_name,b.start_date,b.end_date,b.edb_type FROM predict_edb_conf_calculate_mapping AS a
			INNER JOIN edb_info AS b ON a.from_edb_info_id=b.edb_info_id
			WHERE a.edb_info_id=? AND a.config_id=?  ORDER BY sort ASC `

	err = o.Raw(sql, edbInfoId, configId).Scan(&list).Error
	return
}

func GetPredictEdbConfCalculateMappingDetailListByEdbInfoId(fromEdbInfoIdList []int) (list []*PredictEdbConfCalculateMappingDetail, err error) {
	num := len(fromEdbInfoIdList)
	if num <= 0 {
		return
	}
	o := global.DmSQL["data"]
	sql := ` SELECT a.predict_edb_conf_calculate_mapping_id,a.edb_info_id,a.from_edb_info_id,a.from_edb_code,a.from_source,a.from_source_name,a.sort,a.create_time,a.modify_time,a.from_tag,b.edb_name_source as from_edb_name,b.start_date,b.end_date,b.edb_type,b.edb_code FROM predict_edb_conf_calculate_mapping AS a
			INNER JOIN edb_info AS b ON a.from_edb_info_id=b.edb_info_id
			WHERE a.edb_info_id in (` + utils.GetGormInReplace(num) + `) GROUP BY a.edb_info_id ORDER BY sort ASC `

	err = o.Raw(sql, fromEdbInfoIdList).Scan(&list).Error
	return
}






