package data

import (
	"encoding/json"
	"eta_gn/eta_chart_lib/models"
	"eta_gn/eta_chart_lib/models/data_manage"
	"eta_gn/eta_chart_lib/utils"
	"fmt"
	"io/ioutil"
	"net/http"
	"strings"
)

func AddEdbData(source int, edbCode string) (resp *models.BaseResponse, err error) {
	param := make(map[string]interface{})
	param["EdbCode"] = edbCode
	urlStr := ``
	switch source {
	case utils.DATA_SOURCE_THS:
		urlStr = "/ths/add"
	case utils.DATA_SOURCE_WIND:
		urlStr = "/wind/add"
	case utils.DATA_SOURCE_PB:
		urlStr = "/pb/add"
	case utils.DATA_SOURCE_PB_FINANCE:
		urlStr = "/pb_finance/add"
	case utils.DATA_SOURCE_MANUAL:
		urlStr = "/manual/add"
	case utils.DATA_SOURCE_LZ:
		urlStr = "/lz/add"
	case utils.DATA_SOURCE_YS:
		urlStr = "/smm/add"
	case utils.DATA_SOURCE_GL:
		urlStr = "/mysteel/add"
	case utils.DATA_SOURCE_ZZ:
		urlStr = "/zz/add"
	case utils.DATA_SOURCE_DL:
		urlStr = "/dl/add"
	case utils.DATA_SOURCE_SH:
		urlStr = "/sh/add"
	case utils.DATA_SOURCE_CFFEX:
		urlStr = "/cffex/add"
	case utils.DATA_SOURCE_SHFE:
		urlStr = "/shfe/add"
	case utils.DATA_SOURCE_GIE:
		urlStr = "/gie/add"
	case utils.DATA_SOURCE_LT:
		urlStr = "/lt/add"
	case utils.DATA_SOURCE_COAL:
		urlStr = "/coal/add"
	case utils.DATA_SOURCE_SCI:
		urlStr = "/sci/add"
	case utils.DATA_SOURCE_BAIINFO:
		urlStr = "/baiinfo/add"
	case utils.DATA_SOURCE_NATIONAL_STATISTICS:
		urlStr = "/national_statistics/add"
	default:
	}
	if urlStr == "" {
		err = fmt.Errorf("未实现该指标的刷新接口，请联系管理员")
		return
	}
	resp, err = postRefreshEdbData(param, urlStr)
	return
}

func RefreshEdbData(edbInfoId, source, subSource int, edbCode, startDate string) (resp *models.BaseResponse, err error) {
	param := make(map[string]interface{})
	param["EdbCode"] = edbCode
	param["Source"] = source
	param["EdbInfoId"] = edbInfoId
	param["StartDate"] = startDate
	urlStr := ``
	switch source {
	default:
		edbSource := data_manage.GetEdbSourceBySourceId(source)
		if edbSource != nil {
			urlStr = edbSource.EdbRefreshMethod
		}
	}
	if urlStr == "" {
		err = fmt.Errorf(fmt.Sprint("source:", source, ";未实现该指标的刷新接口，请联系管理员"))
		return
	}
	resp, err = postRefreshEdbData(param, urlStr)
	return
}

func RefreshEdbCalculateData(edbInfoId int, edbCode, startDate string) (resp *models.BaseResponse, err error) {
	param := make(map[string]interface{})
	param["EdbCode"] = edbCode
	param["EdbInfoId"] = edbInfoId
	param["StartDate"] = startDate
	resp, err = postRefreshEdbData(param, "/calculate/refresh")
	return
}

func RefreshPredictEdbCalculateData(edbInfoId int, edbCode, startDate string) (resp *models.BaseResponse, err error) {
	param := make(map[string]interface{})
	param["EdbCode"] = edbCode
	param["EdbInfoId"] = edbInfoId
	param["StartDate"] = startDate
	resp, err = postRefreshEdbData(param, "predict_calculate/refresh")
	return
}

type BaseCalculateResp struct {
	Ret         int
	Msg         string
	ErrMsg      string
	ErrCode     string
	Data        BaseCalculateDataResp
	Success     bool `description:"true 执行成功，false 执行失败"`
	IsSendEmail bool `json:"-" description:"true 发送邮件，false 不发送邮件"`
	IsAddLog    bool `json:"-" description:"true 新增操作日志，false 不新增操作日志" `
}

type BaseCalculateDataResp struct {
	DataMap  map[string]float64
	DateList []string
}

func BaseCalculate(param, lang string) (resp *BaseCalculateResp, err error) {
	urlStr := "calculate/base"
	_, resultByte, err := postAddEdbData(param, urlStr, lang)
	err = json.Unmarshal(resultByte, &resp)
	if err != nil {
		return
	}
	return
}

func postRefreshEdbData(param map[string]interface{}, urlStr string) (resp *models.BaseResponse, err error) {
	postUrl := utils.EDB_LIB_URL + urlStr
	postData, err := json.Marshal(param)
	if err != nil {
		return
	}
	result, err := HttpPost(postUrl, string(postData), "application/json")
	if err != nil {
		return
	}
	err = json.Unmarshal(result, &resp)
	if err != nil {
		return
	}
	return resp, nil
}

func postAddEdbData(paramStr string, urlStr, lang string) (resp *models.BaseResponse, result []byte, err error) {
	postUrl := utils.EDB_LIB_URL + urlStr
	result, err = HttpPost(postUrl, paramStr, lang, "application/json")
	if err != nil {
		return
	}
	err = json.Unmarshal(result, &resp)
	if err != nil {
		return
	}

	return
}

func HttpPost(url, postData string, params ...string) ([]byte, error) {
	body := ioutil.NopCloser(strings.NewReader(postData))
	client := &http.Client{}
	req, err := http.NewRequest("POST", url, body)
	if err != nil {
		return nil, err
	}
	contentType := "application/x-www-form-urlencoded;charset=utf-8"
	if len(params) > 0 && params[0] != "" {
		contentType = params[0]
	}
	req.Header.Set("Content-Type", contentType)
	req.Header.Set("authorization", utils.MD5(utils.APP_EDB_LIB_NAME_EN+utils.EDB_LIB_Md5_KEY))
	resp, err := client.Do(req)
	defer resp.Body.Close()
	b, err := ioutil.ReadAll(resp.Body)
	fmt.Println("HttpPost:" + string(b))
	return b, err
}
